<'
package shr_spi3;

//SPI3 eVC Agent
unit shr_spi3_agent_u like any_unit {
   
    logger : message_logger is instance;
    
    config      : shr_spi3_agent_config_u;   	      // Agent config unit.    

    evc_name    : shr_spi3_evc_name_t;                 //Name of this instance of the eVC, type should be extended

    layer            : shr_spi3_layer_t;               // Environment is modeling either PHY or LINK device
    
    direction        : shr_spi3_direction_t;           // Direction of the agen - RX/TX
				                 
    monitor     : shr_spi3_monitor_u is instance;     // Monitor unit;
    
    data_ports  : list of shr_spi3_port_u is instance;	 // List of data ports, units that provide payload data for the bfm

    active_passive   : erm_active_passive_t; 	      //  ACTIVE/PASSIVE switch
                    
    port_status      : list of shr_spi3_port_stat_s;  //  List of port status structs to hold STPA/DTPA/PTPA status of each port
    
    //Constrain monitor fields
    keep monitor.layer == layer;
    keep monitor.direction == direction;

    //Constrain data ports
    keep data_ports.size() == config.numports;
    keep for each in data_ports {
      it.mode == config.mode;
      it.layer == layer;
      it.port_id == config.active_ports[index];
   };
    
   //Constrain port status
    keep port_status.size() == config.numports;
    keep for each in port_status {
      it.port_id     == config.active_ports[index];
      it.stpa_status == 0;
      it.ptpa_status == 0;
      it.dtpa_status == 0;
   };
    
    event clock;
    event reset_end;

};


//Active TX Link layer agents will have an arbiter instance.
extend ACTIVE'active_passive LINK'layer TX'direction shr_spi3_agent_u {
   arb : shr_spi3_arbiter_u is instance;      //pointer to arbiter
   keep arb.numports == config.numports;
   keep arb.layer == layer;
   keep bfm is a LINK'layer TX'direction shr_spi3_bfm_u (bfm)  => bfm.arb == arb;
};

//All data ports in the LINK TX agent will be of type OUT
extend LINK'layer TX'direction shr_spi3_agent_u {
   keep for each in data_ports {
      it.port_direction == OUT;
   };
};

//ACTIVE PHY layer RX direction agents will have an arbiter instance.
extend ACTIVE'active_passive PHY'layer RX'direction shr_spi3_agent_u {
   arb : shr_spi3_arbiter_u is instance;      //pointer to arbiter
   keep arb.numports == config.numports;
   keep arb.layer == layer;
   keep bfm is a PHY'layer RX'direction shr_spi3_bfm_u (bfm) => bfm.arb == arb;
   
};

//PHY layer RX direction agents will have data ports of type OUT
extend PHY'layer RX'direction shr_spi3_agent_u {
   keep for each in data_ports {
      it.port_direction == OUT;
   };
};

//PHY layer TX direction agents will have data ports of type IN
extend PHY'layer TX'direction shr_spi3_agent_u {
   keep for each in data_ports {
      it.port_direction == IN;
   };
};

//LINK layer RX direction agents will have ports of type IN
extend LINK'layer RX'direction shr_spi3_agent_u {
   keep for each in data_ports {
      it.port_direction == IN;
   };
};


//ACTIVE agents will have a BFM instance
extend ACTIVE'active_passive shr_spi3_agent_u {
   bfm : shr_spi3_bfm_u is instance;
   
   keep bfm.direction == direction;
   keep bfm.layer == layer;
   
   keep bfm.pkt_started.size() == config.numports;
   keep for each in bfm.pkt_started {
      it.port_id == config.active_ports[index];
   };
   
   keep bfm.port_status.size() == config.numports;
   keep for each in bfm.port_status {
      it.port_id == config.active_ports[index];
   };
   
   keep bfm.config == config;
   keep bfm.data_ports == data_ports;
   
};

//All BFMS need a pointer to the parent agent.
extend shr_spi3_bfm_u {
   agent : shr_spi3_agent_u;
   keep agent == get_enclosing_unit(shr_spi3_agent_u);
   
   event clock is only @agent.clock;
   event reset_end is only @agent.reset_end;
};

//All ports need a pointer to the agent
extend shr_spi3_port_u {

    agent       : shr_spi3_agent_u;         // Pointer to agent
    keep mode == agent.config.mode;                  // Narrow/wide bus mode configuration is pulled from the env

    event reset_end is only @agent.reset_end;    

};

//Hook up the agent pointer in the data ports.
extend shr_spi3_agent_u {
    keep for each in data_ports {
      it.agent == me;
   };
   
   run() is also {
   
      messagef(LOW, "\n\n%s SPI3 eVC Agent\n", active_passive);
      messagef(LOW, "Layer : %s\n",layer);
      messagef(LOW, "Direction : %s\n",direction);
      messagef(LOW, "Number of Ports : %d\n",config.numports);
      
   
   };//run is also
};
'>
