<'

package shr_spi3;

unit shr_spi3_bfm_u like any_unit {

   layer       : shr_spi3_layer_t;
   direction   : shr_spi3_direction_t;
   
   config      : shr_spi3_agent_config_u;          //Pointer to agent config
   
   data_ports  : list of shr_spi3_port_u;
   
   xfer_enable : bool;                              // Flag indicating data transfers can be delivered on the bus
   xfer_active : bool;                              // Flag indicating a transfer is active on the SPI-3 bus
   active_port : uint;                              // Data port currently active on bus
   data_count  : uint;                              // Count of data cycles done during the current transfer
   idle_count  : uint;                              // Count of idle cycles done during the current idle period
   next_addr   : uint;                              // Next polled packet available address to send to DUT
   last_renb   : bit;                               // RENB signal value on the previous clock cycle
   pkt_started : list of shr_spi3_bool_s;           // Per-port flags indicating ports with packet transfers started
   port_status : list of shr_spi3_port_stat_s;      // Current packet available FIFO status from each of three sources
   stpa_ports  : list of uint;                      // Queue of port addresses selected for data transfers and STPA status
   ptpa_ports  : list of uint;                      // Queue of port addresses polled for status
   transfers_since_xSX : uint; 	             	    // Number of transfers since last xSX cycle
   next_random_xSX : uint;    	                    // Field to facilitate generating random xSX cycles

   drive_spi3_protocol() @clock is empty;   

   event clock; 
   event reset_end;  
   event stpa_ports_rdy;
   event ptpa_ports_rdy;
   
   //Need to fix this
   event initiate_stpa_evaluation;// is {@stpa_ports_rdy;@env.spi3_in_clock};
   event initiate_ptpa_evaluation;// is {@ptpa_ports_rdy;@env.spi3_in_clock};
   
   event stpa_updated;
   event ptpa_updated;
   event dtpa_updated;
   
   keep xfer_enable        == TRUE;                 // Initially data transfers are enabled (disabling is used to inject dead cycles)
   keep xfer_active        == FALSE;                // Initially there is no transfer active on the bus
   keep active_port        == config.start_port;    // Initialize the active port ID to a port ID guaranteed to be used
   keep data_count         == 0;                    // Initially the count of data cycles is 0
   keep idle_count         == 0;                    // Initially the count of idle cycles is 0
   keep next_addr          == 0;                    // Initialize the next polled packet available address to 0
   keep last_renb          == 1'b0;                 // Default is to assume bus was previously enabled
   keep stpa_ports.size()  == 0;                    // Initially the queue of ports selected for data transfers is empty
   keep ptpa_ports.size()  == 0;                    // Initially the queue of ports polled for FIFO status is empty
   keep for each (stat) in port_status {
       stat.stpa_status == 1'b0;                    // Selected FIFO status initially indicates no space is available
       stat.ptpa_status == 1'b0;                    // Polled FIFO status initially indicates no space is available
       stat.dtpa_status == 1'b0;                    // Direct FIFO status initially indicates no space is available
   };
   keep for each (flag) in pkt_started {
       flag.data == FALSE;                               // Initially no port has started a packet transfer
   };
   
   keep transfers_since_xSX == 0;
   keep next_random_xSX < config.interleave_length;
   
   run() is also {
      start drive_spi3_protocol();
   };
   
};

//Provide a link to the arbiter in the BFM.
extend RX'direction PHY'layer shr_spi3_bfm_u {
   arb	    : shr_spi3_arbiter_u;      
};

extend TX'direction PHY'layer shr_spi3_bfm_u {  
};

extend RX'direction LINK'layer shr_spi3_bfm_u {  
};


//Provide a link to the arbiter in the BFM.
extend TX'direction LINK'layer shr_spi3_bfm_u {
   arb	    : shr_spi3_arbiter_u;
};


'>
