<'
package shr_spi3;

extend shr_spi3_monitor_u {
   agent       : shr_spi3_agent_u;               //Pointer to agent config  
   keep agent == get_enclosing_unit(shr_spi3_agent_u);

   event clock is only @agent.clock;
   event reset_end is only @agent.reset_end;

};

extend shr_spi3_port_stat_s {
    agent : shr_spi3_agent_u;
    keep agent == get_enclosing_unit(shr_spi3_agent_u);
    
    stpa_status_update(updated_status : bit) @agent.clock is {
      wait [agent.config.valid_cycles_after_ENB[port_id]] *cycle;
      stpa_status = updated_status;
    };
    
    dtpa_status_update(updated_status : bit) @agent.clock is {
      wait [agent.config.valid_cycles_after_ENB[port_id]] *cycle;
      dtpa_status = updated_status;
    };
};//extend shr_spi3_port_stat_s

extend LINK'layer RX'direction shr_spi3_monitor_u {

   collect_transactions()@clock is also {
   
      var incoming_datacycle : IN'port_direction shr_spi3_datacycle_s;
      var curr_port : uint(bits:8);
      
      var last_renb : bit = 1;
      
      wait @reset_end;

      if agent is a ACTIVE RX LINK shr_spi3_agent_u (agent) then {
      	 curr_port = agent.bfm.active_port;
      };
      
      while TRUE { //loop forever
         incoming_datacycle = new;
	 if agent is a RX LINK shr_spi3_agent_u (agent) then {
	    if (agent.sig_rval$ == 1 and last_renb == 0) then {  //valid data on the bus
	       incoming_datacycle.sop = agent.sig_rsop$;
	       incoming_datacycle.eop = agent.sig_reop$; 
	       incoming_datacycle.error = agent.sig_rerr$;
	       incoming_datacycle.parity = agent.sig_rprty$;
	       incoming_datacycle.data = agent.sig_rdat$;
	       incoming_datacycle.modulus = agent.sig_rmod$;
	       incoming_datacycle.address = curr_port;
	    
	       recd_cycles.add(deep_copy(incoming_datacycle));
	       emit transfer_received;
               emit agent.data_ports.first(it.port_id == curr_port).transfer_received;
	    }
	    else if (agent.sig_rsx$ == 1 and agent.sig_rval$ == 0) then {//RSX cycle on the bus
	       curr_port = agent.sig_rdat$[7:0];
	    };
	    last_renb = agent.sig_renb$;	    
	 };
	 wait cycle;
   
      };//while TRUE
      
   };//collection_transactions
   
   run() is also {
      start collect_transactions();
   };
   
};  //LINK RX shr_spi3_monitor_u

extend PHY'layer TX'direction shr_spi3_monitor_u {

   collect_transactions()@clock is also {
   
      var incoming_datacycle : IN'port_direction shr_spi3_datacycle_s;
      var curr_port : uint(bits:8);

      wait @reset_end;

      if agent is a ACTIVE TX PHY shr_spi3_agent_u (agent) then {
      	 curr_port = agent.bfm.active_port;
      };
      
      while TRUE { //loop forever
	 incoming_datacycle = new; 
	 if agent is a PHY TX shr_spi3_agent_u (agent) then {
	    if (agent.sig_tenb$ == 0 and agent.sig_tenb.has_z() == FALSE) then {  //valid data on the bus
	       incoming_datacycle.sop = agent.sig_tsop$;
	       incoming_datacycle.eop = agent.sig_teop$; 
	       incoming_datacycle.error = agent.sig_terr$;
	       incoming_datacycle.parity = agent.sig_tprty$;
	       incoming_datacycle.data = agent.sig_tdat$;
	       incoming_datacycle.modulus = agent.sig_tmod$;
	       incoming_datacycle.address = curr_port;
	       last_address = agent.sig_tadr$;
	    
	       recd_cycles.add(deep_copy(incoming_datacycle));
	       emit transfer_received;
               emit agent.data_ports.first(it.port_id == curr_port).transfer_received;	       
	    }
	    else if ((agent.sig_tsx$ == 1 and agent.sig_tsx.has_z() == FALSE) and 
	     (agent.sig_tenb$ == 1 and agent.sig_tenb.has_z() == FALSE)) then {//RSX cycle on the bus
	       curr_port = agent.sig_tdat$[7:0];
	    };
      	 };
	 wait cycle; 
	 
      };//while TRUE
      
   };//collection_transactions
   
   run() is also {
      start collect_transactions();
   };

}; //PHY TX shr_spi3_monitor_u

extend LINK'layer TX'direction shr_spi3_monitor_u {

//primary function of the TX LINK monitor will be to collect data
//from the DTPA, STPA, and PTPA pins

   event stpa_change is change(agent.TX'direction'sig_stpa$)@clock;
   event dtpa_change is change(agent.TX'direction'sig_dtpa$)@clock;
   event curr_port_change;   

   !current_port : uint;

   run() is also {
      current_port = agent.config.active_ports[0];
   };
   
   capture_current_port_id() @clock is {
      if agent is a TX LINK shr_spi3_agent_u (agent) then {
      	 if ((agent.sig_tsx.has_z() == FALSE) and (agent.sig_tsx$ == 1) and (agent.sig_tenb$ == 1)) then {//
	       current_port = agent.sig_tdat$[7:0];
	       emit curr_port_change;
      	 };
      };
   };
   
   run() is also {
      start capture_current_port_id();
   };
      
   evaluate_dtpa() @clock is {
        wait @reset_end;
	var curr_port_id : uint;
        while TRUE {
	 if agent is a TX LINK shr_spi3_agent_u (agent) then {
            for i from 0 to (agent.config.numports - 1) {
	        //set the port id based on the index into the active ports list
	        curr_port_id = agent.config.active_ports[i];
                if (agent.config.enable_dtpa.first(it.port_id == curr_port_id).data) {
                    agent.port_status.first(it.port_id == curr_port_id)
		         .dtpa_status_update(agent.sig_dtpa$[curr_port_id:curr_port_id]);
                };
            };
	 };
            wait @dtpa_change;
       };
    };


   run() is also {
      if agent.config.enable_dtpa.data.has(it == TRUE) {
      	 start evaluate_dtpa();
      };
   };
   
   capture_ptpa() @ clock is {
      
      if agent is a TX LINK shr_spi3_agent_u (agent) then {
      	 var last_addr : uint;
      	 wait @reset_end;
      	 last_addr = agent.sig_tadr$;
      	 wait cycle;
      
      	 while TRUE {
	    if agent.sig_tadr.has_z() == FALSE then {
	       if last_addr in agent.config.active_ports then {
      	          agent.port_status.first(it.port_id == last_addr)
		       .ptpa_status = agent.sig_ptpa$;
      	          //Update the stored value of STPA with the sampled value of PTPA
		  //ONLY when the sampled port (TADR) is NOT the current 'active' port.
		  if last_addr != current_port then {
		     agent.port_status.first(it.port_id == last_addr)
		        .stpa_status = agent.sig_ptpa$;
		  };
	       };
      	       last_addr = agent.sig_tadr$;	       
	    };
	    wait cycle;
      	 };
      };
   };
   run() is also {
      if agent.config.enable_ptpa.data.has(it == TRUE) {
      	 start capture_ptpa();
	 start capture_stpa();
      };
   };
   
   capture_stpa() @ clock is {
   
      if agent is a TX LINK shr_spi3_agent_u (agent) then {
      	 wait @reset_end;
      
      	 while TRUE {
	    start agent.port_status.first(it.port_id == current_port)
	       .stpa_status_update(agent.sig_stpa$);
	    wait @stpa_change or @curr_port_change;
      	 };
      };
   };
};//


'>
