<'
package shr_spi3;

unit shr_spi3_protocol_monitor_u like any_unit{

   layer : shr_spi3_layer_t;
   direction : shr_spi3_direction_t;
   
   spi3_env : shr_spi3_env_u;
   keep spi3_env == get_enclosing_unit(shr_spi3_env_u);
   
   !rx_agent : RX shr_spi3_agent_u;
   !tx_agent : TX shr_spi3_agent_u;
   
   event clock;

   run() is also {
      if spi3_env.spi3_agents.has(it.direction == TX and it.layer == PHY) then {
      	 tx_agent = spi3_env.spi3_agents.first(it.direction == TX and it.layer == PHY).as_a(TX shr_spi3_agent_u);
      }
      else if spi3_env.spi3_agents.has(it.direction == TX and it.layer == LINK) then {
      	 tx_agent = spi3_env.spi3_agents.first(it.direction == TX and it.layer == LINK).as_a(TX shr_spi3_agent_u);
      };
      
      if spi3_env.spi3_agents.has(it.direction == RX and it.layer == LINK) then {
      	 rx_agent = (spi3_env.spi3_agents.first(it.direction == RX and it.layer == LINK)).as_a(RX shr_spi3_agent_u);
      } else if spi3_env.spi3_agents.has(it.direction == RX and it.layer == PHY) then {
      	 rx_agent = spi3_env.spi3_agents.first(it.direction == RX and it.layer == PHY).as_a(RX shr_spi3_agent_u);
      };
      
   };//run is also 

   error_processor(level : shr_spi3_protocol_check_t, message : string) is {
      if level == WARNING then {
      	 set_check(message,WARNING);
      };
      
      if level in [ERROR, WARNING] then {
      	 dut_error(message);
      } else if level == INFO then {
      	 outf("\n %d AVF SPI3:  INFO -- %s\n",sys.time, message);
      };
   };

};//unit shr_spi3_protocol_monitor_u

extend RX shr_spi3_protocol_monitor_u {

   event clock is only @rx_agent.clock;

   rsx_rval_simultaneous_level : shr_spi3_protocol_check_t;
   keep soft rsx_rval_simultaneous_level == ERROR;

   monitor_rsx_rval_simultaneous()@clock is {
      	 while TRUE {
      	    if (rx_agent.sig_rsx$ == 1 and rx_agent.sig_rsx.has_z() == FALSE) and 
	       (rx_agent.sig_rval$ == 1 and rx_agent.sig_rsx.has_z() == FALSE) then {
	       error_processor(rsx_rval_simultaneous_level, 
	          "RSX and RVAL sampled high during the same cycle");
	    };
	    wait cycle;
	 };
   };
   
   rx_bus_values_held_after_renb_level : shr_spi3_protocol_check_t;
   keep soft rx_bus_values_held_after_renb_level == ERROR;
   
   monitor_rx_bus_values_held_after_renb()@clock is {
      
      var last_renb : bit = 1'b1;
      var last_transfer : shr_spi3_datacycle_s = new;
      
      wait @rx_agent.reset_end;
      
      while TRUE {
      
      	 if last_renb == 0 and rx_agent.sig_renb$ == 1 and rx_agent.sig_rval$ == 1 then {
	    //The bus has been paused by reb going high while rval is asserted
	    //and this is the first cycle of "pause", as a result, the values
	    //of the bus should be capture
	    last_transfer.data = rx_agent.sig_rdat$;
	    last_transfer.sop = rx_agent.sig_rsop$;
	    last_transfer.eop = rx_agent.sig_reop$;
	    last_transfer.xfer_start = rx_agent.sig_rsx$;
	 }
	 else if last_renb == 1 and rx_agent.sig_renb$ == 1 and rx_agent.sig_rval$ == 1 then {
	    //The bus is paused, and has been paused for at least one cycle
	    //Need to compare the bus to the captured values to ensure they have not changed.
	    if last_transfer.data != rx_agent.sig_rdat$ or
	       last_transfer.sop != rx_agent.sig_rsop$ or
	       last_transfer.eop != rx_agent.sig_reop$ or
	       last_transfer.xfer_start != rx_agent.sig_rsx$ then {
	          error_processor(rx_bus_values_held_after_renb_level,
		     "Bus values changed while bus paused by RENB.");
	    };
	 };
	 wait cycle;
      };//while TRUE
   };//monitor_rx_bus_values_held_after_renb
   
   rsop_reop_order_level : shr_spi3_protocol_check_t;
   keep soft rsop_reop_order_level == ERROR;
   
   monitor_rsop_reop_order()@clock is {
      var mop : list of bool;
      
      var rsop_message : string = "SOP out of order on port.";
      var reop_message : string = "EOP out of order on port.";
      
      var curr_port : uint;
      var last_renb : uint;
      curr_port = rx_agent.config.start_port;
      last_renb = 0;
      
      for i from 0 to rx_agent.config.num_ports_supported-1 {
      	 mop.add(FALSE);
      };

      	 while TRUE {
      	    if last_renb == 0 and rx_agent.sig_rval$ == 1 then {   
      	       if rx_agent.sig_rsop$ == 1 then {
	          if mop[curr_port] == FALSE then {
	             mop[curr_port] = TRUE;
	          } else {
	             error_processor(rsop_reop_order_level, rsop_message);
	          };
	       };
	       if rx_agent.sig_reop$ == 1 then {
	          if mop[curr_port] == TRUE then {
	             mop[curr_port] = FALSE;
	          } else {
	             error_processor(rsop_reop_order_level, reop_message);
	          };
	       };
	    } else if rx_agent.sig_rval$ == 0 and rx_agent.sig_rsx$ == 1 then {
	       curr_port = rx_agent.sig_rdat$[7:0];
	    };//if rx_agent.sig_renb
            last_renb = rx_agent.sig_renb$;	    
	    wait cycle;
      	 };//while TRUE
   };//monitor_rsop
   
   rx_parity_level : shr_spi3_protocol_check_t;
   keep soft rx_parity_level == ERROR;
   
   monitor_rx_parity()@clock is {
   
      while TRUE {
      
      	 if rx_agent.config.parity == ODD then {
            if (((rx_agent.sig_rval$ == 1 and rx_agent.sig_rval.has_z() == FALSE) or 
                 (rx_agent.sig_rsx$ == 1 and rx_agent.sig_rsx.has_z() == FALSE)) and
                (bitwise_xnor(rx_agent.sig_rdat$) != rx_agent.sig_rprty$)) then {
	       error_processor(rx_parity_level, "Rx Parity Error");
	    };
	 }
	 else if rx_agent.config.parity == EVEN then {
            if (((rx_agent.sig_rval$ == 1 and rx_agent.sig_rval.has_z() == FALSE) or  
                 (rx_agent.sig_rsx$ == 1 and rx_agent.sig_rsx.has_z() == FALSE)) and
                (bitwise_xor(rx_agent.sig_rdat$) != rx_agent.sig_rprty$)) then {
	       error_processor(rx_parity_level, "Rx Parity Error");
	    };
	 };
      
      	 wait cycle;
      };//while TRUE
   
   };//monitor_rx_parity
	       	       
   
   run() is also {
      if rsx_rval_simultaneous_level != DISABLED then {
      	 start monitor_rsx_rval_simultaneous();
      };
      
      if rsop_reop_order_level != DISABLED then {
      	 start monitor_rsop_reop_order();
      };
      
      if rx_bus_values_held_after_renb_level != DISABLED then {
//      	 start monitor_rx_bus_values_held_after_renb();
      };
      
      if rx_parity_level != DISABLED then {
      	 start monitor_rx_parity();
      };
      messagef(LOW, "\n\nSPI3 eVC Rx Agent Protocol Monitor Status:\n");
      messagef(LOW, "RSX/RVAL Asserted Simultaneously - %s\n",rsx_rval_simultaneous_level);
      messagef(LOW, "RSOP/REOP Order Checking - %s\n",rsop_reop_order_level);
      messagef(LOW, "Rx Bus Values Held Correctly While Bus is Paused - %s\n",rx_bus_values_held_after_renb_level);
      messagef(LOW, "Rx Parity Checking - %s\n",rx_parity_level);
      
   };
};//extend RX shr_spi3_protocol_monitor_u

extend TX shr_spi3_protocol_monitor_u {

   event clock is only @tx_agent.clock;

   cycles_after_xTPA_level : shr_spi3_protocol_check_t;
   keep soft cycles_after_xTPA_level == ERROR;

   //Define some events used in determining cycles after xTPA
   
   event active_port_change is true(tx_agent.sig_tsx$ == 1 and
      	             	      	    tx_agent.sig_tenb$ == 1 and
				    tx_agent.sig_tf_rst_l$ == 1)@clock;
   event stpa_fall is fall(tx_agent.sig_stpa$)@clock;
   
   event dtpa_change is change(tx_agent.sig_dtpa$)@clock;
   
   event dtpa_fall;
   
   //Define fields used in determining cycles after xTPA
   active_port : uint;
   keep active_port == 0;
   
   prev_dtpa : uint;
   keep prev_dtpa == 0;
   
   
   //The following method determines when the PHY layer device de-assertes
   // DTPA/STPA and then measures the number of cycles before the LINK
   //layer device de-asserts TENB
   
   on active_port_change { 
      if tx_agent.sig_tdat.has_z() == FALSE then {
      	 active_port = tx_agent.sig_tdat$;
      };
   };

   on dtpa_change {
      if prev_dtpa[active_port:active_port] == 1 and
      	 tx_agent.sig_dtpa$[active_port:active_port] == 0 then {
	    emit dtpa_fall;
      };
      prev_dtpa = tx_agent.sig_dtpa$;
   };
   
   monitor_cycles_after_xTPA()@clock is {
   
      var cycles_since_xtpa : uint = 0;
      
      while TRUE {
      	 wait (@stpa_fall or @dtpa_fall);
	 while tx_agent.sig_tenb$ == 0 {
	    wait cycle;
	    cycles_since_xtpa = cycles_since_xtpa + 1;
	    if cycles_since_xtpa > (tx_agent.config.valid_cycles_after_ENB[active_port] +1) then {
	       error_processor (cycles_after_xTPA_level, 
	       "Number of valid cycles after ENB exceeded.");
	    };//if cycles_since_xtpa......
	 };//while sig_tenb
	 
	 cycles_since_xtpa = 0;

      };//while TRUE
   
   };//monitor_cycles_after_xTPA
   
   tsop_teop_order_level : shr_spi3_protocol_check_t;
   keep soft tsop_teop_order_level == ERROR;
   
   monitor_tsop_teop_order()@clock is {
      var mop : list of bool;
      
      var tsop_message : string = "Tx SOP out of order on port.";
      var teop_message : string = "Tx EOP out of order on port.";
      
      var curr_port : uint;
      curr_port = tx_agent.config.start_port;
      
      wait @tx_agent.reset_end;
      
      for i from 0 to tx_agent.config.num_ports_supported-1 {
      	 mop.add(FALSE);
      };

      	 while TRUE {
      	    if tx_agent.sig_tenb$ == 0 and tx_agent.sig_tenb.has_z() == FALSE then {   
      	       if tx_agent.sig_tsop$ == 1 and tx_agent.sig_tsop.has_z() == FALSE then {
	          if mop[curr_port] == FALSE then {
	             mop[curr_port] = TRUE;
	          } else {
	             error_processor(tsop_teop_order_level, tsop_message);
	          };
	       };
	       if tx_agent.sig_teop$ == 1 and tx_agent.sig_teop.has_z() == FALSE then {
	          if mop[curr_port] == TRUE then {
	             mop[curr_port] = FALSE;
	          } else {
	             error_processor(tsop_teop_order_level, teop_message);
	          };
	       };
	    } else if (tx_agent.sig_tenb$ == 1 and tx_agent.sig_tenb.has_z() == FALSE) and 
	              (tx_agent.sig_tsx$ == 1 and tx_agent.sig_tsx.has_z() == FALSE) then {
	       curr_port = tx_agent.sig_tdat$[7:0];
	    };//if rx_agent.sig_tenb
	    wait cycle;
      	 };//while TRUE
   };//monitor_tsop

   tx_parity_level : shr_spi3_protocol_check_t;
   keep soft tx_parity_level == ERROR;
   
   monitor_tx_parity()@clock is {
   
      while TRUE {
      
      	 if tx_agent.config.parity == ODD then {
            if (((tx_agent.sig_tenb$ == 0 and tx_agent.sig_tenb.has_z() == FALSE) or 
                 (tx_agent.sig_tsx$ == 1 and tx_agent.sig_tsx.has_z() == FALSE)) and
                (bitwise_xnor(tx_agent.sig_tdat$) != tx_agent.sig_tprty$)) then {
               error_processor(tx_parity_level, "Tx Parity Error");
	    };
	 }
	 else if tx_agent.config.parity == EVEN then {
            if (((tx_agent.sig_tenb$ == 0 and tx_agent.sig_tenb.has_z() == FALSE) or 
                 (tx_agent.sig_tsx$ == 1 and tx_agent.sig_tsx.has_z() == FALSE)) and
                (bitwise_xor(tx_agent.sig_tdat$) != tx_agent.sig_tprty$)) then {
	       error_processor(tx_parity_level, "Tx Parity Error");
	    };
	 };
	 
	 wait cycle;
      
      };//while TRUE
   
   };//monitor_tx_parity
   
   run() is also {
      if cycles_after_xTPA_level != DISABLED then {
      	 start monitor_cycles_after_xTPA();
      };
      
      if tsop_teop_order_level != DISABLED then {
      	 start monitor_tsop_teop_order();
      };
      
      if tx_parity_level != DISABLED then {
      	 start monitor_tx_parity();
      };
      
      messagef(LOW, "\n\nSPI3 eVC Tx Agent Protocol Monitor Status:\n");
      messagef(LOW, "Number of valid cycles after xTPA - %s\n",cycles_after_xTPA_level);
      messagef(LOW, "TSOP/TEOP Order Checking - %s\n",tsop_teop_order_level);
      messagef(LOW, "Tx Parity Checking - %s\n",tx_parity_level);
      
   };//run() is also 
};//extend TX shr_spi3_protocol_monitor_u   

extend shr_spi3_env_u {
   protocol_monitors : list of shr_spi3_protocol_monitor_u is instance;
   keep soft protocol_monitors.size() == 2;
   keep soft protocol_monitors[0].direction == RX;
   keep soft protocol_monitors[1].direction == TX;
};


'>
