
<'
package shr_spi3;

//Arbiter result data structure.
struct shr_spi3_arb_result_s like any_sequence_item {
    port_found : bool;                    // Flag indicating that a valid port ID was found
    port_id    : uint;                    // Port identifier for next port to service
};

//Port status data structure containing STPA, DTPA, and PTPA information.
struct shr_spi3_port_stat_s like any_sequence_item {
    port_id : uint;
    stpa_status : bit;                    // Selected packet available port status
    ptpa_status : bit;                    // Polled packet available port status
    dtpa_status : bit;                    // Direct packet available port status
};

//Structure with port ID and boolean.
struct shr_spi3_bool_s like any_sequence_item {
    port_id : uint;
    data : bool;
};

//Data structure returned by the data port to the BFM sequence
struct shr_spi3_inport_bfmdata_s like any_sequence_item {
    data  : uint(bits:32);                // Data sent to BFM
    mod   : uint(bits:2);                 // Modulus for number of valid data bytes (sent to DUT)
    eop   : bit;
    sop   : bit;
    error : bool;    	      	          // Flag indicating the current packet is errored    
};

//BFM sequence item.
struct shr_spi3_datacycle_s like any_sequence_item {
    port_direction : shr_spi3_port_direction;  //Indicate the direction relative to the
      	             	      	               //'e' code (i.e. IN is data from the DUT
					       //to the eVC.					   
    payload_kind : shr_spi3_payload_kind_t;
    %xfer_start : bit;                    // Transfer start flag to/from the SPI bus
    %enable     : bit;                    // Enable indicator to/from the SPI bus
    %valid      : bit;                    // Valid bit to/from the SPI bus
    %sop        : bit;                    // Start of packet flag to/from the SPI bus
    %eop        : bit;                    // End of packet flag to/from the SPI bus
    %error      : bit;                    // Error flag to/from the SPI bus
    %parity     : bit;                    // Even/odd parity bit to/from the SPI bus
    %data       : uint(bits:32);          // Data to/from the SPI bus
    %modulus    : uint(bits:2);           // Valid byte modulus to/from the SPI bus
    %address    : uint;                   // Address used for polled FIFO status
    %stpa       : bit;
    %ptpa       : bit;
    %dtpa       : uint;
    
    bad_parity  : bool;       	          //Flag to signal if xfer should be generated
      	             	      	          //with bad parity
    keep soft bad_parity == FALSE;
    
    valid_clocks : uint;      	         //The number of clocks for which
      	             	      	          //the transfer is valid
    keep soft valid_clocks == 1;
    
    
};

//SPI3 Packet
struct shr_spi3_packet_s like any_sequence_item {
   %port       : uint(bits : 8);
   %data       : list of byte;
   keep data.size() > 0;
};

//Addr to be driven to the DUT.
struct shr_spi3_xtpa_s like any_sequence_item {
   %addr : uint;
};

//Port num struct.
struct shr_spi3_next_port_s like any_sequence_item {
   %port       :uint(bits:8);
};


'>

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=
// MODIFICATION HISTORY :
// -=-=-=-=-=-=-=-=-=-=-
//
// $Log: shr_spi3_struct.e,v $
// Revision 1.3  2004/04/05 17:37:46  mgraham
// Added a 'port_direction' field to the datacycle struct.
//
// Revision 1.2  2004/04/02 21:25:10  mgraham
// Added bad_parity bool to transfer, contrained to be false.
//
// Revision 1.1.1.1  2004/03/18 16:10:43  mgraham
// Intial checkin.
//
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=
